Imports System.Xml

Public Class frmMain

    ''' <summary>
    ''' Create a new Xml file.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnCreate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCreate.Click

        ' Set up the XmlWriter settings.
        Dim settings As New XmlWriterSettings()
        settings.Indent = True

        ' Initialize the XmlWriter.
        Dim XmlWrt As XmlWriter = XmlWriter.Create(Application.StartupPath & "XmlTest.xml", settings)

        With XmlWrt
            ' Write the Xml declaration.
            .WriteStartDocument()
            ' Write a comment.
            .WriteComment("Xml example.")

            ' Write the root element.
            .WriteStartElement("People")

            ' Start our first person.
            .WriteStartElement("Person")
            ' Give this person the attribute ID of 1.
            .WriteAttributeString("ID", "1")

            ' The person nodes.
            .WriteStartElement("Name")
            .WriteString("Bob")
            .WriteEndElement()

            .WriteStartElement("Age")
            .WriteString("28")
            .WriteEndElement()

            .WriteStartElement("Weight")
            .WriteString("180")
            .WriteEndElement()
            ' The end of this person.
            .WriteEndElement()

            ' Next person.
            .WriteStartElement("Person")
            .WriteAttributeString("ID", "2")

            .WriteStartElement("Name")
            .WriteString("Julie")
            .WriteEndElement()

            .WriteStartElement("Age")
            .WriteString("34")
            .WriteEndElement()

            .WriteStartElement("Weight")
            .WriteString("125")
            ' The end of this person.
            .WriteEndElement()
            ' The end of People.
            .WriteEndElement()

            ' Close the XmlWriter.
            .WriteEndDocument()
            .Close()
        End With

        ShowXml()

    End Sub

    ''' <summary>
    ''' Edit an existing Xml file.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnEdit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnEdit.Click

        ' Load the XmlDocument.
        Dim xd As New XmlDocument()
        xd.Load(Application.StartupPath & "XmlTest.xml")

        ' Find the node where the Person's attribute ID is 1.
        Dim nod As XmlNode = xd.SelectSingleNode("/People/Person[@ID='1']")
        If nod IsNot Nothing Then
            ' Change the InnerText of the age node.
            nod.ChildNodes(1).InnerText = "29"
        Else
            ' Where's Bob?
            MessageBox.Show("Couldn't find Bob.", "Where's Bob?", MessageBoxButtons.OK, _
                MessageBoxIcon.Information)
        End If

        ' Save the Xml.
        xd.Save(Application.StartupPath & "XmlTest.xml")

        ShowXml()

    End Sub

    ''' <summary>
    ''' Add a node to an existing Xml file.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAdd.Click

        ' Create our new element as a string.
        Dim newPerson As String = _
            "<Person ID=""3"">" & Environment.NewLine & " " & _
            "   <Name>Tommy</Name>" & Environment.NewLine & " " & _
            "   <Age>62</Age>" & Environment.NewLine & " " & _
            "   <Weight>190</Weight>" & Environment.NewLine & "  " & _
            "</Person>"

        ' Load the XmlDocument.
        Dim xd As New XmlDocument()
        xd.Load(Application.StartupPath & "XmlTest.xml")

        ' Create a new XmlDocumentFragment for our document.
        Dim docFrag As XmlDocumentFragment = xd.CreateDocumentFragment()
        ' The Xml for this fragment is our newPerson string.
        docFrag.InnerXml = newPerson
        ' Append the new person to the root element.
        Dim root As XmlNode = xd.DocumentElement
        root.AppendChild(docFrag)

        ' Save the Xml.
        xd.Save(Application.StartupPath & "XmlTest.xml")

        ShowXml()

    End Sub

    ''' <summary>
    ''' Remove a node from an existing Xml file.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnRemove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRemove.Click

        ' Load the XmlDocument.
        Dim xd As New XmlDocument()
        xd.Load(Application.StartupPath & "XmlTest.xml")

        ' Find Bob's node with the attribute ID of 1
        ' using the XPath again.
        Dim nod As XmlNode = xd.SelectSingleNode("/People/Person[@ID='1']")
        If nod IsNot Nothing Then
            ' Since we found Bob's node, we will remove
            ' it and all of his information.
            nod.ParentNode.RemoveChild(nod)
        Else
            ' Where's Bob?
            MessageBox.Show("Couldn't find Bob.", "Where's Bob?", MessageBoxButtons.OK, _
                MessageBoxIcon.Information)
        End If

        ' Save the Xml.
        xd.Save(Application.StartupPath & "XmlTest.xml")

        ShowXml()

    End Sub

    ''' <summary>
    ''' "Merge" two XML files.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnMerge_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMerge.Click

        ' Create the Xml file to merge from.
        ' Of course, skip this part in your case. Your XML file that you want to 
        ' add from will already exist.
        Dim settings As New XmlWriterSettings()
        settings.Indent = True

        Dim XmlWrt As XmlWriter = XmlWriter.Create(Application.StartupPath & "XmlTest2.xml", settings)

        With XmlWrt
            ' Write the Xml declaration.
            .WriteStartDocument()
            ' Write a comment.
            .WriteComment("Xml example.")

            ' Write the root element.
            .WriteStartElement("People")

            ' Start our first person.
            .WriteStartElement("Person")
            ' Give this person the attribute ID of 1.
            .WriteAttributeString("ID", "4")

            ' The person nodes.
            .WriteStartElement("Name")
            .WriteString("Bill")
            .WriteEndElement()

            .WriteStartElement("Age")
            .WriteString("37")
            .WriteEndElement()

            .WriteStartElement("Weight")
            .WriteString("162")
            .WriteEndElement()
            ' The end of this person.
            .WriteEndElement()

            ' Next person.
            .WriteStartElement("Person")
            .WriteAttributeString("ID", "5")

            .WriteStartElement("Name")
            .WriteString("Amy")
            .WriteEndElement()

            .WriteStartElement("Age")
            .WriteString("19")
            .WriteEndElement()

            .WriteStartElement("Weight")
            .WriteString("105")
            ' The end of this person.
            .WriteEndElement()
            ' The end of People.
            .WriteEndElement()

            ' Close the XmlWriter.
            .WriteEndDocument()
            .Close()
        End With

        ' The actual "merge". 
        ' The XmlDocument that we want to merge from.
        Dim xdFrom As New XmlDocument()
        xdFrom.Load(Application.StartupPath & "XmlTest2.xml")

        ' The XMLDocument that we want to merge to.
        Dim xdTo As New XmlDocument()
        xdTo.Load(Application.StartupPath & "XmlTest.xml")

        ' Loop through all of the nodes in the "from" document.
        ' We don't want to copy the root node in this instance.
        For Each nod As XmlNode In xdFrom.DocumentElement.ChildNodes
            ' Import the node to our "to" document.
            Dim tmpNod As XmlNode = xdTo.ImportNode(nod, True)
            ' Append this temporary node to the end of the "to" document
            ' but inside the root element.
            xdTo.DocumentElement.AppendChild(tmpNod)
        Next nod

        ' Save the "to" document with it's newly appended nodes.
        xdTo.Save(Application.StartupPath & "XmlTest.xml")

        ShowXml()

    End Sub

    ''' <summary>
    ''' Read the XML file.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnRead_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRead.Click

        ' Settings for the XmlReader.
        Dim XmlRdSets As New XmlReaderSettings()
        XmlRdSets.IgnoreComments = True
        XmlRdSets.IgnoreWhitespace = True

        ' StringBuilder to hold the results
        ' for this example.
        Dim sb As New System.Text.StringBuilder()

        ' Create the XmlReader.
        Using XmlRd As XmlReader = XmlReader.Create(Application.StartupPath & "XmlTest.xml", XmlRdSets)

            With XmlRd
                ' Read the root element.
                .ReadStartElement("People")

                Do
                    ' Get the attribute of the person.
                    sb.Append("ID: " & .GetAttribute("ID"))
                    ' Read the person's start element.
                    .ReadStartElement("Person")

                    ' The Name value.
                    sb.Append(" Name: " & .ReadElementString())
                    ' The Age value.
                    sb.Append(" Age: " & .ReadElementString())
                    ' The Weight value.
                    sb.Append(" Weight: " & .ReadElementString() & Environment.NewLine)

                    ' Read the end of this person.
                    .ReadEndElement()

                    ' Loop to read the next person if available.
                    If .Name <> "Person" Then
                        Exit Do
                    End If
                Loop

                ' Read the end of the People element and
                ' therefore the end of the file.
                .ReadEndElement()
            End With
        End Using

        ' Show the results.
        MessageBox.Show(sb.ToString(), "Reading XML Results", MessageBoxButtons.OK, MessageBoxIcon.None)

    End Sub

    ''' <summary>
    ''' Show the Xml in the WebBrowser.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ShowXml()

        Me.WebBrowser1.Navigate(Application.StartupPath & "XmlTest.xml")

    End Sub

End Class
